#include <ffts.h>
#include <signal.h>
#include <fract.h>

#include <sport.h>
#include <circ.h>

extern	int	Real_Out[256], Imag_Out[256];
extern 	int Buffer0[], Buffer1[];

// these are used to split the 512 pt buffers into 2, 256 pt buffers
int * Buffer0_1;
int * Buffer1_1;

// double-buffering toggle flag
int	db_tgl = 0;

// Do_FFT flag
int	DoFFT=0;

int max = 1;
int index = 1;
int	Bit_Reversal_On = 0;


void 	SPORT0_Receive_Int( int sig_int )
{

}

main()
{
	int i;
	// set up pointers which point to the middle of our two buffers
	Buffer0_1 = Buffer0 + 256;
	Buffer1_1 = Buffer1 + 256;

	// Set up serial port 0

	// setup 10 Mhz internal SCLK with 80Mhz clock in
	sport0.sclkdiv = COMPUTE_SCLKDIV(80000000, 10000000);

	// set RFS to be generated ever 20 SCLK periods
	sport0.rfsdiv = 20;

	// setup sport0 control register
	sport0.control.single.isclk 	= 1;	// internal sclk
	sport0.control.single.irfs		= 1;	// Internal RFS
	sport0.control.single.rfsr		= 1;	// RFS required
	sport0.control.single.slen		= 15;	// 16 bit words

	// setup autobuffering regisers
	sport0.autobuffer.receive_enable 	= 1;	// enable receive autobuffering on sport0
	sport0.autobuffer.rmreg			 	= 1;	// M3 (=1 by default)
	sport0.autobuffer.rireg			 	= 2;	// I0 (we need to reserve this)

	// Set up DAG register i2 to be used for autobuffering and to point to _Buffer0 initially
	// with a length of 512
	// this macro is found in circ.h
	circ_setup(2, Buffer0, 512);

	// start sport 0
	sport_start(0);

	// this interrupt will be generated every time the DAG pointer used for autobuffering wraps
	interrupt(SIGSPORT0RECV, SPORT0_Receive_Int); 

	for(;;)
	{
		asm("idle;");
		if (DoFFT)
		{
			// if we're here, we've received a complete buffer of data
			
			DoFFT = 0;
			if (!db_tgl)
			{
	
				/* 	Disable Autobuffering and let the SPORT ISR know that we are currently not using autobuffering
					by setting the Bit_Reversal_On value to a 1 */

				Bit_Reversal_On = 1;
				sport0.autobuffer.receive_enable 	= 0;	// disable  receive autobuffering on sport0

				fft256(Buffer1, Buffer1_1, Real_Out, Imag_Out);

				/* 	Reenable Autobuffering and the SPORT ISR know that we are currently using autobuffering
					by setting the Bit_Reversal_On value to a 0 */

				sport0.autobuffer.receive_enable 	= 1;	// enable receive autobuffering on sport0
				Bit_Reversal_On = 0;

			}
			else
			{
				Bit_Reversal_On = 1;
				sport0.autobuffer.receive_enable 	= 0;	// disable  receive autobuffering on sport0

				fft256(Buffer0, Buffer0_1, Real_Out, Imag_Out);
				
				sport0.autobuffer.receive_enable 	= 1;	// enable receive autobuffering on sport0
				Bit_Reversal_On = 0;
			}

			// do FFT magic.... ? split them up


			// Calculate magnitude and find maxes
			max = 0;
			for (i=0;i<128;i++)
			{
				Real_Out[i] = Real_Out[i]*Real_Out[i] + Imag_Out[i]*Imag_Out[i];
				if (Real_Out[i] > max)
				{
					max = Real_Out[i];
					index = i;
				}
			}
		
			// perform interpolation to find sub-bin maxima here
		}
	}
} 
